import { __read } from 'tslib';
import { Injectable, InjectionToken, Directive, ElementRef, Input, Output, EventEmitter, forwardRef, HostListener, Inject, Renderer2, NgModule } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';

var FlatpickrDefaults = /** @class */ (function () {
    function FlatpickrDefaults() {
        this.altFormat = 'F j, Y';
        this.altInput = false;
        this.altInputClass = '';
        this.allowInput = false;
        this.appendTo = undefined;
        this.clickOpens = true;
        this.dateFormat = 'Y-m-d';
        this.disable = [];
        this.disableMobile = false;
        this.enable = [];
        this.enableTime = false;
        this.enableSeconds = false;
        this.hourIncrement = 1;
        this.inline = false;
        this.maxDate = undefined;
        this.minDate = undefined;
        this.minuteIncrement = 5;
        this.mode = 'single';
        this.nextArrow = '>';
        this.noCalendar = false;
        this.prevArrow = '<';
        this.shorthandCurrentMonth = false;
        this.static = false;
        this.time24hr = false;
        this.utc = false;
        this.weekNumbers = false;
        this.wrap = false;
        this.plugins = [];
        this.convertModelValue = false;
    }
    return FlatpickrDefaults;
}());
FlatpickrDefaults.decorators = [
    { type: Injectable },
];
var FLATPICKR = new InjectionToken('flatpickr');
var FLATPICKR_CONTROL_VALUE_ACCESSOR = {
    provide: NG_VALUE_ACCESSOR,
    useExisting: forwardRef(function () { return FlatpickrDirective; }),
    multi: true
};
var FlatpickrDirective = /** @class */ (function () {
    function FlatpickrDirective(elm, defaults, renderer, flatpickrFactory) {
        this.elm = elm;
        this.defaults = defaults;
        this.renderer = renderer;
        this.flatpickrFactory = flatpickrFactory;
        this.flatpickrReady = new EventEmitter();
        this.flatpickrChange = new EventEmitter();
        this.flatpickrValueUpdate = new EventEmitter();
        this.flatpickrOpen = new EventEmitter();
        this.flatpickrClose = new EventEmitter();
        this.flatpickrMonthChange = new EventEmitter();
        this.flatpickrYearChange = new EventEmitter();
        this.flatpickrDayCreate = new EventEmitter();
        this.isDisabled = false;
        this.onChangeFn = function () { };
        this.onTouchedFn = function () { };
    }
    FlatpickrDirective.prototype.ngAfterViewInit = function () {
        var _this = this;
        var options = {
            altFormat: this.altFormat,
            altInput: this.altInput,
            altInputClass: this.altInputClass,
            allowInput: this.allowInput,
            appendTo: this.appendTo,
            clickOpens: this.clickOpens,
            dateFormat: this.dateFormat,
            disable: this.disable,
            disableMobile: this.disableMobile,
            enable: this.enable,
            enableTime: this.enableTime,
            enableSeconds: this.enableSeconds,
            hourIncrement: this.hourIncrement,
            defaultDate: this.initialValue,
            inline: this.inline,
            maxDate: this.maxDate,
            minDate: this.minDate,
            minuteIncrement: this.minuteIncrement,
            mode: this.mode,
            nextArrow: this.nextArrow,
            noCalendar: this.noCalendar,
            parseDate: this.parseDate,
            prevArrow: this.prevArrow,
            shorthandCurrentMonth: this.shorthandCurrentMonth,
            static: this.static,
            time24hr: this.time24hr,
            weekNumbers: this.weekNumbers,
            getWeek: this.getWeek,
            wrap: this.wrap,
            plugins: this.plugins,
            onChange: function (selectedDates, dateString, instance) {
                _this.flatpickrChange.emit({ selectedDates: selectedDates, dateString: dateString, instance: instance });
            },
            onOpen: function (selectedDates, dateString, instance) {
                _this.flatpickrOpen.emit({ selectedDates: selectedDates, dateString: dateString, instance: instance });
            },
            onClose: function (selectedDates, dateString, instance) {
                _this.flatpickrClose.emit({ selectedDates: selectedDates, dateString: dateString, instance: instance });
            },
            onMonthChange: function (selectedDates, dateString, instance) {
                _this.flatpickrMonthChange.emit({ selectedDates: selectedDates, dateString: dateString, instance: instance });
            },
            onYearChange: function (selectedDates, dateString, instance) {
                _this.flatpickrYearChange.emit({ selectedDates: selectedDates, dateString: dateString, instance: instance });
            },
            onReady: function (selectedDates, dateString, instance) {
                _this.flatpickrReady.emit({ selectedDates: selectedDates, dateString: dateString, instance: instance });
            },
            onValueUpdate: function (selectedDates, dateString, instance) {
                _this.flatpickrValueUpdate.emit({ selectedDates: selectedDates, dateString: dateString, instance: instance });
            },
            onDayCreate: function (selectedDates, dateString, instance, dayElement) {
                _this.flatpickrDayCreate.emit({
                    selectedDates: selectedDates,
                    dateString: dateString,
                    instance: instance,
                    dayElement: dayElement
                });
            }
        };
        if (this.locale) {
            options.locale = this.locale;
        }
        Object.keys(options).forEach(function (key) {
            if (typeof options[key] === 'undefined') {
                options[key] = ((_this)).defaults[key];
            }
        });
        options.time_24hr = options.time24hr;
        this.instance = this.flatpickrFactory(this.elm.nativeElement, options);
        this.setDisabledState(this.isDisabled);
    };
    FlatpickrDirective.prototype.ngOnChanges = function (changes) {
        var _this = this;
        if (this.instance) {
            Object.keys(changes).forEach(function (inputKey) {
                _this.instance.set((inputKey), ((_this))[inputKey]);
            });
        }
    };
    FlatpickrDirective.prototype.ngOnDestroy = function () {
        this.instance.destroy();
    };
    FlatpickrDirective.prototype.writeValue = function (value) {
        var convertedValue = value;
        if (this.convertModelValue && this.mode === 'range' && value) {
            convertedValue = [value.from, value.to];
        }
        if (this.instance) {
            this.instance.setDate(convertedValue);
        }
        else {
            this.initialValue = convertedValue;
        }
    };
    FlatpickrDirective.prototype.registerOnChange = function (fn) {
        this.onChangeFn = fn;
    };
    FlatpickrDirective.prototype.registerOnTouched = function (fn) {
        this.onTouchedFn = fn;
    };
    FlatpickrDirective.prototype.setDisabledState = function (isDisabled) {
        this.isDisabled = isDisabled;
        if (this.instance) {
            if (this.isDisabled) {
                this.renderer.setProperty(this.instance._input, 'disabled', 'disabled');
            }
            else {
                this.renderer.removeAttribute(this.instance._input, 'disabled');
            }
        }
    };
    FlatpickrDirective.prototype.inputChanged = function () {
        var _this = this;
        var value = this.elm.nativeElement.value;
        if (this.convertModelValue && typeof value === 'string') {
            switch (this.mode) {
                case 'multiple':
                    var dates = value
                        .split('; ')
                        .map(function (str) { return _this.instance.parseDate(str, _this.instance.config.dateFormat, !_this.instance.config.enableTime); });
                    this.onChangeFn(dates);
                    break;
                case 'range':
                    var _a = __read(value
                        .split(this.instance.l10n.rangeSeparator)
                        .map(function (str) { return _this.instance.parseDate(str, _this.instance.config.dateFormat, !_this.instance.config.enableTime); }), 2), from = _a[0], to = _a[1];
                    this.onChangeFn({ from: from, to: to });
                    break;
                case 'single':
                default:
                    this.onChangeFn(this.instance.parseDate(value, this.instance.config.dateFormat, !this.instance.config.enableTime));
            }
        }
        else {
            this.onChangeFn(value);
        }
    };
    return FlatpickrDirective;
}());
FlatpickrDirective.decorators = [
    { type: Directive, args: [{
                selector: '[mwlFlatpickr]',
                providers: [FLATPICKR_CONTROL_VALUE_ACCESSOR],
                host: {
                    '(blur)': 'onTouchedFn()'
                }
            },] },
];
FlatpickrDirective.ctorParameters = function () { return [
    { type: ElementRef, },
    { type: FlatpickrDefaults, },
    { type: Renderer2, },
    { type: undefined, decorators: [{ type: Inject, args: [FLATPICKR,] },] },
]; };
FlatpickrDirective.propDecorators = {
    "altFormat": [{ type: Input },],
    "altInput": [{ type: Input },],
    "altInputClass": [{ type: Input },],
    "allowInput": [{ type: Input },],
    "appendTo": [{ type: Input },],
    "clickOpens": [{ type: Input },],
    "dateFormat": [{ type: Input },],
    "disable": [{ type: Input },],
    "disableMobile": [{ type: Input },],
    "enable": [{ type: Input },],
    "enableTime": [{ type: Input },],
    "enableSeconds": [{ type: Input },],
    "hourIncrement": [{ type: Input },],
    "inline": [{ type: Input },],
    "maxDate": [{ type: Input },],
    "minDate": [{ type: Input },],
    "minuteIncrement": [{ type: Input },],
    "mode": [{ type: Input },],
    "nextArrow": [{ type: Input },],
    "noCalendar": [{ type: Input },],
    "parseDate": [{ type: Input },],
    "prevArrow": [{ type: Input },],
    "shorthandCurrentMonth": [{ type: Input },],
    "static": [{ type: Input },],
    "time24hr": [{ type: Input },],
    "weekNumbers": [{ type: Input },],
    "getWeek": [{ type: Input },],
    "wrap": [{ type: Input },],
    "plugins": [{ type: Input },],
    "locale": [{ type: Input },],
    "convertModelValue": [{ type: Input },],
    "flatpickrReady": [{ type: Output },],
    "flatpickrChange": [{ type: Output },],
    "flatpickrValueUpdate": [{ type: Output },],
    "flatpickrOpen": [{ type: Output },],
    "flatpickrClose": [{ type: Output },],
    "flatpickrMonthChange": [{ type: Output },],
    "flatpickrYearChange": [{ type: Output },],
    "flatpickrDayCreate": [{ type: Output },],
    "inputChanged": [{ type: HostListener, args: ['input',] },],
};
var USER_DEFAULTS = new InjectionToken('flatpickr defaults');
function defaultsFactory(userDefaults) {
    var defaults = new FlatpickrDefaults();
    Object.assign(defaults, userDefaults);
    return defaults;
}
var FlatpickrModule = /** @class */ (function () {
    function FlatpickrModule() {
    }
    FlatpickrModule.forRoot = function (flatpickr, userDefaults) {
        if (userDefaults === void 0) { userDefaults = {}; }
        return {
            ngModule: FlatpickrModule,
            providers: [
                flatpickr,
                {
                    provide: USER_DEFAULTS,
                    useValue: userDefaults
                },
                {
                    provide: FlatpickrDefaults,
                    useFactory: defaultsFactory,
                    deps: [USER_DEFAULTS]
                }
            ]
        };
    };
    return FlatpickrModule;
}());
FlatpickrModule.decorators = [
    { type: NgModule, args: [{
                declarations: [FlatpickrDirective],
                exports: [FlatpickrDirective]
            },] },
];

export { FlatpickrDefaults, USER_DEFAULTS, defaultsFactory, FlatpickrModule, FLATPICKR, FLATPICKR_CONTROL_VALUE_ACCESSOR as ɵa, FlatpickrDirective as ɵb };
//# sourceMappingURL=angularx-flatpickr.js.map
